@@ -20,12 +20,15 @@
  */
 
 #include "config.h"
-#include <gnome.h>
+
+#include <glib/gi18n.h>
 
 #include "seahorse-context.h"
 #include "seahorse-unknown-source.h"
 #include "seahorse-unknown-key.h"
 
+#include "common/seahorse-registry.h"
+
 enum {
     PROP_0,
     PROP_KEY_TYPE,
@@ -33,16 +36,16 @@ enum {
     PROP_LOCATION
 };
 
-G_DEFINE_TYPE (SeahorseUnknownSource, seahorse_unknown_source, SEAHORSE_TYPE_KEY_SOURCE);
+G_DEFINE_TYPE (SeahorseUnknownSource, seahorse_unknown_source, SEAHORSE_TYPE_SOURCE);
 
 /* -----------------------------------------------------------------------------
  * INTERNAL
  */
 
 static void
-search_done (SeahorseOperation *op, SeahorseKey *skey)
+search_done (SeahorseOperation *op, SeahorseObject *sobj)
 {
-    skey->location = SKEY_LOC_UNKNOWN;
+	g_object_set (sobj, "location", SEAHORSE_LOCATION_MISSING, NULL);
 }
 
 /* -----------------------------------------------------------------------------
@@ -50,54 +53,42 @@ search_done (SeahorseOperation *op, SeahorseKey *skey)
  */
 
 static SeahorseOperation*
-seahorse_unknown_source_load (SeahorseKeySource *src, GQuark keyid)
+seahorse_unknown_source_load (SeahorseSource *src, GQuark id)
 {
     SeahorseUnknownSource *usrc;
     
     g_assert (SEAHORSE_IS_UNKNOWN_SOURCE (src));
     usrc = SEAHORSE_UNKNOWN_SOURCE (src);
     
-    if (keyid) {
-        g_return_val_if_fail (keyid, NULL);
-        seahorse_unknown_source_add_key (usrc, keyid, NULL);
+    if (id) {
+        g_return_val_if_fail (id, NULL);
+        seahorse_unknown_source_add_object (usrc, id, NULL);
     }
     
     return seahorse_operation_new_complete (NULL);
 }
 
-static void
-seahorse_unknown_source_stop (SeahorseKeySource *src)
-{
-
-}
-
-static guint
-seahorse_unknown_source_get_state (SeahorseKeySource *src)
-{
-    return 0;
-}
-
 static SeahorseOperation* 
-seahorse_unknown_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
+seahorse_unknown_source_import (SeahorseSource *sksrc, GInputStream *input)
 {
     g_return_val_if_reached (NULL);
     return NULL;
 }
 
 static SeahorseOperation* 
-seahorse_unknown_source_export_raw (SeahorseKeySource *sksrc, GSList *keyids, 
-                                    gpgme_data_t data)
+seahorse_unknown_source_export_raw (SeahorseSource *sksrc, GSList *ids, 
+                                    GOutputStream *output)
 {
     g_return_val_if_reached (NULL);
     return NULL;
 }
 
 static gboolean            
-seahorse_unknown_source_remove (SeahorseKeySource *sksrc, SeahorseKey *skey,
+seahorse_unknown_source_remove (SeahorseSource *sksrc, SeahorseObject *sobj,
                                 guint name, GError **error)
 {
-    g_return_val_if_fail (sksrc == seahorse_key_get_source (skey), FALSE);
-    seahorse_key_destroy (skey);
+    g_return_val_if_fail (sksrc == seahorse_object_get_source (sobj), FALSE);
+    seahorse_context_remove_object (SCTX_APP (), sobj);
     return TRUE;
 }
 
@@ -128,7 +119,7 @@ seahorse_unknown_source_get_property (GObject *object, guint prop_id, GValue *va
         g_value_set_string (value, _("Unavailable Key"));
         break;
     case PROP_LOCATION:
-        g_value_set_uint (value, SKEY_LOC_UNKNOWN);
+        g_value_set_uint (value, SEAHORSE_LOCATION_MISSING);
         break;
     }
 }
@@ -143,7 +134,7 @@ static void
 seahorse_unknown_source_class_init (SeahorseUnknownSourceClass *klass)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-    SeahorseKeySourceClass *parent_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
+    SeahorseSourceClass *parent_class = SEAHORSE_SOURCE_CLASS (klass);
    
     seahorse_unknown_source_parent_class = g_type_class_peek_parent (klass);
     
@@ -151,8 +142,6 @@ seahorse_unknown_source_class_init (SeahorseUnknownSourceClass *klass)
     gobject_class->get_property = seahorse_unknown_source_get_property;
     
     parent_class->load = seahorse_unknown_source_load;
-    parent_class->stop = seahorse_unknown_source_stop;
-    parent_class->get_state = seahorse_unknown_source_get_state;
     parent_class->import = seahorse_unknown_source_import;
     parent_class->export_raw = seahorse_unknown_source_export_raw;
     parent_class->remove = seahorse_unknown_source_remove;
@@ -166,8 +155,10 @@ seahorse_unknown_source_class_init (SeahorseUnknownSourceClass *klass)
                              NULL, G_PARAM_READABLE));
 
     g_object_class_install_property (gobject_class, PROP_LOCATION,
-        g_param_spec_uint ("location", "Key Location", "Where the key is stored. See SeahorseKeyLoc", 
-                           0, G_MAXUINT, SKEY_LOC_INVALID, G_PARAM_READABLE));    
+        g_param_spec_uint ("location", "Key Location", "Where the key is stored. See SeahorseLocation", 
+                           0, G_MAXUINT, SEAHORSE_LOCATION_MISSING, G_PARAM_READABLE));    
+    
+	seahorse_registry_register_type (NULL, SEAHORSE_TYPE_UNKNOWN_SOURCE, "key-source", NULL);
 }
 
 /* -----------------------------------------------------------------------------
@@ -180,24 +171,24 @@ seahorse_unknown_source_new (GQuark ktype)
    return g_object_new (SEAHORSE_TYPE_UNKNOWN_SOURCE, "key-type", ktype, NULL);
 }
 
-SeahorseKey*                     
-seahorse_unknown_source_add_key (SeahorseUnknownSource *usrc, GQuark keyid,
-                                 SeahorseOperation *search)
+SeahorseObject*                     
+seahorse_unknown_source_add_object (SeahorseUnknownSource *usrc, GQuark id,
+                                    SeahorseOperation *search)
 {
-    SeahorseKey *skey;
+    SeahorseObject *sobj;
 
-    g_return_val_if_fail (keyid != 0, NULL);
+    g_return_val_if_fail (id != 0, NULL);
 
-    skey = seahorse_context_get_key (SCTX_APP (), SEAHORSE_KEY_SOURCE (usrc), keyid);
-    if (!skey) {
-        skey = SEAHORSE_KEY (seahorse_unknown_key_new (usrc, keyid));
-        seahorse_context_add_key (SCTX_APP (), skey);
+    sobj = seahorse_context_get_object (SCTX_APP (), SEAHORSE_SOURCE (usrc), id);
+    if (!sobj) {
+        sobj = SEAHORSE_OBJECT (seahorse_unknown_key_new (usrc, id));
+        seahorse_context_add_object (SCTX_APP (), sobj);
     }
     
     if (search) {
-        skey->location = SKEY_LOC_SEARCHING;
-        seahorse_operation_watch (search, G_CALLBACK (search_done), NULL, skey);
+        sobj->_location = SEAHORSE_LOCATION_SEARCHING;
+        seahorse_operation_watch (search, (SeahorseDoneFunc) search_done, sobj, NULL, NULL);
     }
     
-    return skey;
+    return sobj;
 }
