@@ -21,6 +21,23 @@
  */
 
 #include "config.h"
+
+#include "seahorse-util.h"
+#include "seahorse-key.h"
+#include "seahorse-gconf.h"
+
+#include "pgp/seahorse-pgp-module.h"
+
+#include "ssh/seahorse-ssh-module.h"
+
+#include <gio/gio.h>
+#include <glib/gstdio.h>
+
+#ifdef WITH_SHARING
+#include <avahi-glib/glib-watch.h>
+#include <avahi-glib/glib-malloc.h>
+#endif 
+
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -28,81 +45,47 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
-
-#include <gnome.h>
-#include <libgnomevfs/gnome-vfs.h>
-
-#ifdef WITH_SHARING
-#include <avahi-glib/glib-watch.h>
-#include <avahi-glib/glib-malloc.h>
-#endif 
-
-#include "seahorse-gpgmex.h"
-#include "seahorse-util.h"
-#include "seahorse-key.h"
-#include "seahorse-pgp-key.h"
-#include "seahorse-gconf.h"
-#include "seahorse-vfs-data.h"
-
-#ifdef WITH_SSH
-#include "seahorse-ssh-key.h"
-#endif
+#include <errno.h>
+#include <glib/gi18n.h>
 
 static const gchar *bad_filename_chars = "/\\<>|";
 
 void
-seahorse_util_show_error (GtkWindow *parent, const gchar *heading, const gchar *message)
+seahorse_util_show_error (GtkWidget *parent, const gchar *heading, const gchar *message)
 {
 	GtkWidget *error;
-    gchar *t;
+	gchar *text;
     
-	g_return_if_fail (message);
+	g_return_if_fail (message || heading);
+	if (!message)
+		message = "";
     
-    if (heading)
-        t = g_strconcat("<big><b>", heading, "</b></big>\n\n", message, NULL);
-    else
-        t = g_strdup (message);
+	if (heading)
+		text = g_strconcat("<big><b>", heading, "</b></big>\n\n", message, NULL);
+	else
+		text = g_strdup (message);
 	
-	error = gtk_message_dialog_new_with_markup (parent, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
-		GTK_BUTTONS_CLOSE, t);
-    g_free (t);
+	if (parent) {
+		if (!GTK_IS_WIDGET (parent)) {
+			g_warn_if_reached ();
+			parent = NULL;
+		} else {
+			if (!GTK_IS_WINDOW (parent)) 
+				parent = gtk_widget_get_toplevel (parent);
+			if (!GTK_IS_WINDOW (parent) && GTK_WIDGET_TOPLEVEL (parent))
+				parent = NULL;
+		}
+	}
+	
+	error = gtk_message_dialog_new_with_markup (GTK_WINDOW (parent), 
+	                                            GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
+	                                            GTK_BUTTONS_CLOSE, text);
+	g_free (text);
 	
 	gtk_dialog_run (GTK_DIALOG (error));
 	gtk_widget_destroy (error);
 }
 
-/**
- * seahorse_util_handle_gpgme:
- * @err: Error value
- *
- * Shows an error dialog for @err.
- **/
-void
-seahorse_util_handle_gpgme (gpgme_error_t err, const char* desc, ...)
-{
-    gchar *t = NULL;
-
-    switch (gpgme_err_code(err)) {
-    case GPG_ERR_CANCELED:
-    case GPG_ERR_NO_ERROR:
-    case GPG_ERR_ECANCELED:
-        return;
-    default: 
-        break;
-    }
-
-    va_list ap;
-    va_start(ap, desc);
-  
-    if (desc) 
-        t = g_strdup_vprintf (desc, ap);
-
-    va_end(ap);
-        
-    seahorse_util_show_error (NULL, t, gpgme_strerror (err));
-    g_free(t);
-}
-
 void
 seahorse_util_handle_error (GError* err, const char* desc, ...)
 {
@@ -122,21 +105,30 @@ seahorse_util_handle_error (GError* err, const char* desc, ...)
     
     seahorse_util_show_error (NULL, t, err->message ? err->message : "");
     g_free(t);
-    
-    g_clear_error (&err);
 }    
 
-/**
- * seahorse_util_gpgme_error_domain
- * Returns: The GError domain for gpgme errors
- **/
-GQuark
-seahorse_util_gpgme_error_domain ()
+gboolean
+seahorse_util_prompt_delete (const gchar *text)
 {
-    static GQuark q = 0;
-    if(q == 0)
-        q = g_quark_from_static_string ("seahorse-gpgme-error");
-    return q;
+	GtkWidget *warning, *button;
+	gint response;
+	
+	warning = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+	                                  GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
+	                                  "%s", text);
+    
+	button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
+	gtk_dialog_add_action_widget (GTK_DIALOG (warning), GTK_WIDGET (button), GTK_RESPONSE_REJECT);
+	gtk_widget_show (button);
+	
+	button = gtk_button_new_from_stock(GTK_STOCK_DELETE);
+	gtk_dialog_add_action_widget (GTK_DIALOG (warning), GTK_WIDGET (button), GTK_RESPONSE_ACCEPT);
+	gtk_widget_show (button);
+	
+	response = gtk_dialog_run (GTK_DIALOG (warning));
+	gtk_widget_destroy (warning);
+	
+	return (response == GTK_RESPONSE_ACCEPT);
 }
 
 /**
@@ -152,29 +144,6 @@ seahorse_util_error_domain ()
     return q;
 }
 
-/**
- * seahorse_util_gpgme_to_error
- * @gerr GPGME error
- * @err  Resulting GError object
- **/
-void    
-seahorse_util_gpgme_to_error (gpgme_error_t gerr, GError** err)
-{
-    gpgme_err_code_t code;
-    
-    /* Make sure this is actually an error */
-    g_assert (!GPG_IS_OK(gerr));
-    code = gpgme_err_code (gerr);
-    
-    /* Special case some error messages */
-    if (code == GPG_ERR_DECRYPT_FAILED) {
-        g_set_error (err, SEAHORSE_GPGME_ERROR, code, "%s", 
-                     _("Decryption failed. You probably do not have the decryption key."));
-    } else {
-        g_set_error (err, SEAHORSE_GPGME_ERROR, code, "%s", 
-                     gpgme_strerror (gerr));
-    }
-}
 /** 
  * seahorse_util_get_date_string:
  * @time: Time value to parse
@@ -193,7 +162,7 @@ seahorse_util_get_date_string (const time_t time)
 		return "0";
 	
 	created_date = g_date_new ();
-	g_date_set_time (created_date, time);
+	g_date_set_time_t (created_date, time);
 	created_string = g_new (gchar, 11);
 	g_date_strftime (created_string, 11, "%Y-%m-%d", created_date);
 	return created_string;
@@ -217,7 +186,7 @@ seahorse_util_get_display_date_string (const time_t time)
 		return "0";
 	
 	created_date = g_date_new ();
-	g_date_set_time (created_date, time);
+	g_date_set_time_t (created_date, time);
 	created_string = g_new (gchar, 11);
 	g_date_strftime (created_string, 11, _("%Y-%m-%d"), created_date);
 	return created_string;
@@ -250,7 +219,7 @@ seahorse_util_set_text_view_string (GtkTextView *view, GString *string)
 }
 
 /**
- * seahorse_util_write_data_to_text:
+ * seahorse_util_read_to_text:
  * @data: Data to write
  * @len: Length of the data
  *
@@ -259,29 +228,33 @@ seahorse_util_set_text_view_string (GtkTextView *view, GString *string)
  * Returns: The string read from data
  **/
 gchar*
-seahorse_util_write_data_to_text (gpgme_data_t data, guint *len)
+seahorse_util_read_to_text (GInputStream *input, guint *len)
 {
-    gint size = 128;
-    gchar *buffer, *text;
-    guint nread = 0;
-    GString *string;
+	gint size = 128;
+	gchar *buffer, *text;
+	gsize nread = 0;
+	GString *string;
+	GSeekable *seek;
 
-    gpgme_data_seek (data, 0, SEEK_SET);
+	if (G_IS_SEEKABLE (input)) {
+		seek = G_SEEKABLE (input);
+		g_seekable_seek (seek, 0, SEEK_SET, NULL, NULL);
+	}
 
-    string = g_string_new ("");
-    buffer = g_new (gchar, size);
+	string = g_string_new ("");
+	buffer = g_new (gchar, size);
     
-    while ((nread = gpgme_data_read (data, buffer, size)) > 0)
-        string = g_string_append_len (string, buffer, nread);
+	while (g_input_stream_read_all (input, buffer, size, &nread, NULL, NULL) && nread == size)
+		string = g_string_append_len (string, buffer, nread);
 
-    if (len)
-        *len = string->len;
+	if (len)
+		*len = string->len;
     
-    text = string->str;
-    g_string_free (string, FALSE);
-    g_free (buffer);
-    
-    return text;
+	text = string->str;
+	g_string_free (string, FALSE);
+	g_free (buffer);
+	
+	return text;
 }
 
 /** 
@@ -290,23 +263,24 @@ seahorse_util_write_data_to_text (gpgme_data_t data, guint *len)
  * Breaks out one block of data (usually a key)
  * 
  * @buf: A string buffer to write the data to.
- * @data: The GPGME data block to read from.
+ * @input: The input stream to read from.
  * @start: The start signature to look for.
  * @end: The end signature to look for.
  * 
  * Returns: The number of bytes copied.
  */
 guint
-seahorse_util_read_data_block (GString *buf, gpgme_data_t data, 
+seahorse_util_read_data_block (GString *buf, GInputStream *input, 
                                const gchar *start, const gchar* end)
 {
     const gchar *t;
     guint copied = 0;
     gchar ch;
+    gsize read;
      
     /* Look for the beginning */
     t = start;
-    while (gpgme_data_read (data, &ch, 1) == 1) {
+    while (g_input_stream_read_all (input, &ch, 1, &read, NULL, NULL) && read == 1) {
         
         /* Match next char */            
         if (*t == ch)
@@ -322,7 +296,7 @@ seahorse_util_read_data_block (GString *buf, gpgme_data_t data,
     
     /* Look for the end */
     t = end;
-    while (gpgme_data_read (data, &ch, 1) == 1) {
+    while (g_input_stream_read_all (input, &ch, 1, &read, NULL, NULL) && read == 1) {
         
         /* Match next char */
         if (*t == ch)
@@ -339,6 +313,36 @@ seahorse_util_read_data_block (GString *buf, gpgme_data_t data,
     return copied;
 }
 
+gsize
+seahorse_util_memory_output_length (GMemoryOutputStream *output)
+{
+	GSeekable *seekable;
+	goffset offset, end;
+	
+	/* 
+	 * This is a replacement for g_memory_output_stream_get_data_size()
+	 * which is not available in current version of glib.
+	 */
+	
+	g_return_val_if_fail (G_IS_MEMORY_OUTPUT_STREAM (output), 0);
+	g_return_val_if_fail (G_IS_SEEKABLE (output), 0);
+	
+	seekable = G_SEEKABLE (output);
+	offset = g_seekable_tell (seekable);
+	
+	if (!g_seekable_seek (seekable, 0, G_SEEK_END, NULL, NULL))
+		g_return_val_if_reached (0);
+	
+	end = g_seekable_tell (seekable);
+	
+	if (offset != end) {
+		if (!g_seekable_seek (seekable, offset, G_SEEK_SET, NULL, NULL))
+			g_return_val_if_reached (0);
+	}
+	
+	return (gsize)end;
+}
+
 /** 
  * seahorse_util_printf_fd:
  * @fd: The file descriptor to write to
@@ -483,16 +487,16 @@ seahorse_util_uri_split_last (gchar* uri)
 gboolean
 seahorse_util_uri_exists (const gchar* uri)
 {
-    GnomeVFSURI* vuri;    
-    gboolean exists;
+	GFile *file;
+	gboolean exists;
 
-    vuri = gnome_vfs_uri_new (uri);
-    g_return_val_if_fail (vuri != NULL, FALSE);
-    
-    exists = gnome_vfs_uri_exists (vuri);
-    gnome_vfs_uri_unref (vuri);
-    
-    return exists;
+	file = g_file_new_for_uri (uri);
+	g_return_val_if_fail (file, FALSE);
+	
+	exists = g_file_query_exists (file, NULL);
+	g_object_unref (file);
+	
+	return exists;
 }       
     
 /**
@@ -597,78 +601,6 @@ seahorse_util_uri_replace_ext (const gchar *uri, const gchar *ext)
     return ret;
 }
 
-/* Context for callback below */
-typedef struct _VisitUriCtx
-{
-    GArray* files;
-    const gchar* base_uri;
-}
-VisitUriCtx;
-
-/* Called for each sub file or directory */
-static gboolean
-visit_uri (const gchar *rel_path, GnomeVFSFileInfo *info, gboolean recursing_will_loop,
-                gpointer data, gboolean *recurse)
-{
-    VisitUriCtx* ctx = (VisitUriCtx*)data;
-    gchar* t = g_strconcat (ctx->base_uri, "/", rel_path, NULL);
-    gchar* uri = gnome_vfs_make_uri_canonical (t);
-    g_free (t);
-
-    if(info->type != GNOME_VFS_FILE_TYPE_DIRECTORY)
-        g_array_append_val (ctx->files, uri);
-
-    *recurse = !recursing_will_loop;
-    return TRUE;
-}
-
-/**
- * seahorse_util_uris_expand
- * @uris: The null-terminated vector of URIs to enumerate
- * 
- * Find all files in the given set of uris.                                         
- * 
- * Returns: Newly allocated null-terminated string vector of URIs.
- */
-gchar** 
-seahorse_util_uris_expand (const gchar** uris)
-{
-    GnomeVFSFileInfo* info;
-    GArray* files  = NULL;
-    const gchar** u;
-    gchar* uri;
-
-    files = g_array_new (TRUE, FALSE, sizeof(gchar*));    
-    info = gnome_vfs_file_info_new ();
-    
-    for(u = uris; *u; u++) {
-
-        uri = gnome_vfs_make_uri_canonical (*u);
-
-        if (gnome_vfs_get_file_info (uri, info, GNOME_VFS_FILE_INFO_DEFAULT) == GNOME_VFS_OK &&
-            info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) 
-        {
-                VisitUriCtx ctx;
-                ctx.files = files;
-                ctx.base_uri = uri;
-                
-                gnome_vfs_directory_visit (uri, GNOME_VFS_FILE_INFO_DEFAULT, 
-                    GNOME_VFS_DIRECTORY_VISIT_DEFAULT | GNOME_VFS_DIRECTORY_VISIT_LOOPCHECK,
-                    visit_uri, &ctx);
-               
-        /* Not a directory */     
-        } else {
-           
-            g_array_append_val (files, uri);
-            uri = NULL; /* To prevent freeing below */
-        }
-        
-        g_free (uri);
-    }            
-    
-    return (gchar**)g_array_free (files, FALSE);            
-}
-
 /**
  * seahorse_util_uris_package
  * @package: Package uri
@@ -689,41 +621,51 @@ seahorse_util_uris_package (const gchar* package, const char** uris)
     gchar *cmd;
     gchar *t;
     gchar *x;
-    GnomeVFSFileInfo *info;
-    GnomeVFSResult result;
+    GFile *file, *fpackage;
     
-    t = gnome_vfs_get_local_path_from_uri (package);
-    x = g_shell_quote(t);
-    g_free(t);
+    	fpackage = g_file_new_for_uri (package);
+    	t = g_file_get_path (fpackage);
+    	x = g_shell_quote (t);
+    	g_free (t);
     
-    /* create execution */
-    str = g_string_new ("");
-    g_string_printf (str, "file-roller --add-to=%s", x);
-    g_free(x);
+    	/* create execution */
+    	str = g_string_new ("");
+    	g_string_printf (str, "file-roller --add-to=%s", x);
+    	g_free (x);
     
-    while(*uris) {
-        /* We should never be passed any remote uris at this point */
-        x = gnome_vfs_make_uri_canonical (*uris);
-        
-        t = gnome_vfs_get_local_path_from_uri (x);
-        g_free(x);
-        
-        g_return_val_if_fail (t != NULL, FALSE);
+    	while(*uris) {
+    		/* We should never be passed any remote uris at this point */
+    		x = g_uri_parse_scheme (*uris);
+    		if (x) 
+    			file = g_file_new_for_uri (*uris);
+    		else
+    			file = g_file_new_for_path (*uris);
+    		g_free (x);
+    		
+    		t = g_file_get_path (file);
+    		g_object_unref (file);
+    		g_return_val_if_fail (t != NULL, FALSE);
 
-        x = g_shell_quote(t);
-        g_free(t);
+    		x = g_shell_quote (t);
+    		g_free (t);
 
-        g_string_append_printf (str, " %s", x);
-        g_free(x);
+    		g_string_append_printf (str, " %s", x);
+    		g_free (x);
         
-        uris++;
-    }
+    		uris++;
+    	}
         
-    /* Execute the command */
-    cmd = g_string_free (str, FALSE);
-    r = g_spawn_command_line_sync (cmd, &out, NULL, &status, &err);
-    g_free (cmd); 
+    	/* Execute the command */
+    	cmd = g_string_free (str, FALSE);
+    	r = g_spawn_command_line_sync (cmd, &out, NULL, &status, &err);
+    	g_free (cmd); 
     
+    	/* TODO: This won't work for remote packages if we support them in the future */
+    	t = g_file_get_path (fpackage);
+	g_chmod (t, S_IRUSR | S_IWUSR);
+	g_free (t);
+    	g_object_unref (fpackage);
+
     if (out) {
         g_print (out);
         g_free (out);
@@ -731,6 +673,7 @@ seahorse_util_uris_package (const gchar* package, const char** uris)
     
     if (!r) {
         seahorse_util_handle_error (err, _("Couldn't run file-roller"));
+        g_clear_error (&err);
         return FALSE;
     }
     
@@ -740,67 +683,62 @@ seahorse_util_uris_package (const gchar* package, const char** uris)
         return FALSE;
     }
     
-    info = gnome_vfs_file_info_new ();
-	info->permissions = GNOME_VFS_PERM_USER_READ | GNOME_VFS_PERM_USER_WRITE;
-	result = gnome_vfs_set_file_info (package, info, GNOME_VFS_SET_FILE_INFO_PERMISSIONS);
-    gnome_vfs_file_info_unref (info);
-	    
-	if (result != GNOME_VFS_OK) {
-    	seahorse_util_handle_error (err, _("Couldn't set permissions on backup file."));
-        return FALSE;
-    }
-
     return TRUE;
 }
 
 GQuark
 seahorse_util_detect_mime_type (const gchar *mime)
 {
-    if (!mime || g_ascii_strcasecmp (mime, GNOME_VFS_MIME_TYPE_UNKNOWN) == 0) {
-        g_warning ("couldn't get mime type for data");
-        return 0;
-    }
+	if (!mime || g_ascii_strcasecmp (mime, "application/octet-stream") == 0) {
+		g_warning ("couldn't get mime type for data");
+		return 0;
+	}
 
-    if (g_ascii_strcasecmp (mime, "application/pgp-keys") == 0)
-        return SKEY_PGP;
+	if (g_ascii_strcasecmp (mime, "application/pgp-encrypted") == 0 ||
+	    g_ascii_strcasecmp (mime, "application/pgp-keys") == 0)
+		return SEAHORSE_PGP;
     
 #ifdef WITH_SSH 
-    /* TODO: For now all PEM keys are treated as SSH keys */
-    else if (g_ascii_strcasecmp (mime, "application/x-ssh-key") == 0 ||
-             g_ascii_strcasecmp (mime, "application/x-pem-key") == 0)
-        return SKEY_SSH;
+	/* TODO: For now all PEM keys are treated as SSH keys */
+	else if (g_ascii_strcasecmp (mime, "application/x-ssh-key") == 0 ||
+	         g_ascii_strcasecmp (mime, "application/x-pem-key") == 0)
+		return SEAHORSE_SSH;
 #endif 
     
-    g_warning ("unsupported type of key data: %s", mime);
-    return 0;
+	g_warning ("unsupported type of key data: %s", mime);
+	return 0;
 }
 
 GQuark
 seahorse_util_detect_data_type (const gchar *data, guint length)
 {
-    const gchar* mime = gnome_vfs_get_mime_type_for_data (data, length);
-    return seahorse_util_detect_mime_type (mime);
+	gboolean uncertain;
+	gchar *mime;
+	GQuark type;
+	
+	mime = g_content_type_guess (NULL, (const guchar*)data, length, &uncertain);
+	g_return_val_if_fail (mime, 0);
+	
+	type = seahorse_util_detect_mime_type (mime);
+	g_free (mime);
+	
+	return type;
 }
 
 GQuark
 seahorse_util_detect_file_type (const gchar *uri)
 {
-    GnomeVFSResult res;
-    GnomeVFSFileInfo *info;
-    GQuark ret = 0;
-    
-    info = gnome_vfs_file_info_new ();
-    res = gnome_vfs_get_file_info (uri, info, 
-            GNOME_VFS_FILE_INFO_GET_MIME_TYPE | GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE);
-    
-    if (res == GNOME_VFS_OK)
-        ret = seahorse_util_detect_mime_type (info->mime_type);
-    else
-        g_warning ("couldn't get mime type for: %s: %s", uri, 
-                   gnome_vfs_result_to_string (res));
-
-    gnome_vfs_file_info_unref (info);
-    return ret;
+	gboolean uncertain;
+	gchar *mime;
+	GQuark type;
+	
+	mime = g_content_type_guess (uri, NULL, 0, &uncertain);
+	g_return_val_if_fail (mime, 0);
+	
+	type = seahorse_util_detect_mime_type (mime);
+	g_free (mime);
+	
+	return type;
 }
 
 gboolean
@@ -812,39 +750,39 @@ seahorse_util_write_file_private (const gchar* filename, const gchar* contents,
     return ret;
 }
 
-GtkWidget*
+GtkDialog*
 seahorse_util_chooser_save_new (const gchar *title, GtkWindow *parent)
 {
-    GtkWidget *dialog;
+    GtkDialog *dialog;
     
-    dialog = gtk_file_chooser_dialog_new (title, 
+    dialog = GTK_DIALOG (gtk_file_chooser_dialog_new (title, 
                 parent, GTK_FILE_CHOOSER_ACTION_SAVE, 
                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                 GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-                NULL);
+                NULL));
 
-    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+    gtk_dialog_set_default_response (dialog, GTK_RESPONSE_ACCEPT);
     gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
     return dialog;
 }
 
-GtkWidget*
+GtkDialog*
 seahorse_util_chooser_open_new (const gchar *title, GtkWindow *parent)
 {
-    GtkWidget *dialog;
+    GtkDialog *dialog;
     
-    dialog = gtk_file_chooser_dialog_new (title, 
+    dialog = GTK_DIALOG (gtk_file_chooser_dialog_new (title, 
                 parent, GTK_FILE_CHOOSER_ACTION_OPEN, 
                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
-                NULL);
+                NULL));
 
-    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
+    gtk_dialog_set_default_response (dialog, GTK_RESPONSE_ACCEPT);
     gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (dialog), FALSE);
     return dialog;
 }
 void
-seahorse_util_chooser_show_key_files (GtkWidget *dialog)
+seahorse_util_chooser_show_key_files (GtkDialog *dialog)
 {
     GtkFileFilter* filter = gtk_file_filter_new ();
     
@@ -867,7 +805,7 @@ seahorse_util_chooser_show_key_files (GtkWidget *dialog)
 }
 
 void
-seahorse_util_chooser_show_archive_files (GtkWidget *dialog)
+seahorse_util_chooser_show_archive_files (GtkDialog *dialog)
 {
     GtkFileFilter* filter;
     int i;
@@ -907,7 +845,7 @@ seahorse_util_chooser_show_archive_files (GtkWidget *dialog)
 }
 
 void
-seahorse_util_chooser_set_filename (GtkWidget *dialog, GList *keys)
+seahorse_util_chooser_set_filename_full (GtkDialog *dialog, GList *keys)
 {
     gchar *t = NULL;
     
@@ -917,14 +855,22 @@ seahorse_util_chooser_set_filename (GtkWidget *dialog, GList *keys)
         g_free (t);
     }
 }
+
+void
+seahorse_util_chooser_set_filename (GtkDialog *dialog, SeahorseKey *skey)
+{
+	GList *keys = g_list_append (NULL, skey);
+	seahorse_util_chooser_set_filename_full (dialog, keys);
+	g_list_free (keys);
+}
     
 gchar*      
-seahorse_util_chooser_save_prompt (GtkWidget *dialog)
+seahorse_util_chooser_save_prompt (GtkDialog *dialog)
 {
     GtkWidget* edlg;
     gchar *uri = NULL;
     
-    while (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
+    while (gtk_dialog_run (dialog) == GTK_RESPONSE_ACCEPT) {
      
         uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER (dialog));
 
@@ -954,19 +900,19 @@ seahorse_util_chooser_save_prompt (GtkWidget *dialog)
             break;
     }
   
-    gtk_widget_destroy (dialog);
+    gtk_widget_destroy (GTK_WIDGET (dialog));
     return uri;
 }
 
 gchar*      
-seahorse_util_chooser_open_prompt (GtkWidget *dialog)
+seahorse_util_chooser_open_prompt (GtkDialog *dialog)
 {
     gchar *uri = NULL;
     
-    if(gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+    if(gtk_dialog_run (dialog) == GTK_RESPONSE_ACCEPT)
         uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
-        
-    gtk_widget_destroy (dialog);
+
+    gtk_widget_destroy (GTK_WIDGET (dialog));
     return uri;
 }
 
@@ -1008,7 +954,7 @@ seahorse_util_check_suffix (const gchar *path, SeahorseSuffix suffix)
 gchar*
 seahorse_util_add_suffix (const gchar *path, SeahorseSuffix suffix, const gchar *prompt)
 {
-    GtkWidget *dialog;
+    GtkDialog *dialog;
     const gchar *ext;
     gchar *uri;
     gchar *t;
@@ -1033,6 +979,7 @@ seahorse_util_add_suffix (const gchar *path, SeahorseSuffix suffix, const gchar
         uri = NULL;
             
         uri = seahorse_util_chooser_save_prompt (dialog);
+        gtk_widget_destroy (GTK_WIDGET (dialog));
     }
 
     return uri;         
@@ -1050,7 +997,7 @@ seahorse_util_add_suffix (const gchar *path, SeahorseSuffix suffix, const gchar
 gchar*
 seahorse_util_remove_suffix (const gchar *path, const gchar *prompt)
 {
-    GtkWidget *dialog;
+    GtkDialog *dialog;
     gchar *uri;
     gchar *t;
     
@@ -1070,6 +1017,7 @@ seahorse_util_remove_suffix (const gchar *path, const gchar *prompt)
         uri = NULL;
             
         uri = seahorse_util_chooser_save_prompt (dialog);
+        gtk_widget_destroy (GTK_WIDGET (dialog));
     }   
     
     return uri;
@@ -1104,46 +1052,16 @@ seahorse_util_strvec_length (const gchar **vec)
     return len;
 }
 
-/* TODO: This function no longer makes sense once we 
-   have multiple key and encryption types */
-gpgme_key_t* 
-seahorse_util_keylist_to_keys (GList *keys)
-{
-    gpgme_key_t *recips;
-    int i;
-    
-    recips = g_new0 (gpgme_key_t, g_list_length (keys) + 1);
-    
-    for (i = 0; keys != NULL; keys = g_list_next (keys), i++) {
-        g_return_val_if_fail (SEAHORSE_IS_PGP_KEY (keys->data), recips);
-        recips[i] = SEAHORSE_PGP_KEY (keys->data)->pubkey;
-        gpgmex_key_ref (recips[i]);
-    }
-    
-    return recips;
-}
-
-void
-seahorse_util_free_keys (gpgme_key_t* keys)
-{
-    gpgme_key_t* k = keys;
-    if (!keys)
-        return;
-    while (*k)
-        gpgmex_key_unref (*(k++));
-    g_free (keys);
-}
-
 static gint 
-sort_keys_by_source (SeahorseKey *k1, SeahorseKey *k2)
+sort_objects_by_source (SeahorseObject *k1, SeahorseObject *k2)
 {
-    SeahorseKeySource *sk1, *sk2;
+    SeahorseSource *sk1, *sk2;
     
-    g_assert (SEAHORSE_IS_KEY (k1));
-    g_assert (SEAHORSE_IS_KEY (k2));
+    g_assert (SEAHORSE_IS_OBJECT (k1));
+    g_assert (SEAHORSE_IS_OBJECT (k2));
     
-    sk1 = seahorse_key_get_source (k1);
-    sk2 = seahorse_key_get_source (k2);
+    sk1 = seahorse_object_get_source (k1);
+    sk2 = seahorse_object_get_source (k2);
     
     if (sk1 == sk2)
         return 0;
@@ -1152,24 +1070,24 @@ sort_keys_by_source (SeahorseKey *k1, SeahorseKey *k2)
 }
 
 GList*        
-seahorse_util_keylist_sort (GList *keys)
+seahorse_util_objects_sort (GList *objects)
 {
-    return g_list_sort (keys, (GCompareFunc)sort_keys_by_source);
+    return g_list_sort (objects, (GCompareFunc)sort_objects_by_source);
 }
 
 GList*       
-seahorse_util_keylist_splice (GList *keys)
+seahorse_util_objects_splice (GList *objects)
 {
-    SeahorseKeySource *psk = NULL;
-    SeahorseKeySource *sk;
+    SeahorseSource *psk = NULL;
+    SeahorseSource *sk;
     GList *prev = NULL;
     
-    /* Note that the keylist must be sorted */
+    /* Note that the objects must be sorted */
     
-    for ( ; keys; keys = g_list_next (keys)) {
+    for ( ; objects; objects = g_list_next (objects)) {
      
-        g_return_val_if_fail (SEAHORSE_IS_KEY (keys->data), NULL);
-        sk = seahorse_key_get_source (SEAHORSE_KEY (keys->data));
+        g_return_val_if_fail (SEAHORSE_IS_OBJECT (objects->data), NULL);
+        sk = seahorse_object_get_source (SEAHORSE_OBJECT (objects->data));
         
         /* Found a disconuity */
         if (psk && sk != psk) {
@@ -1179,11 +1097,11 @@ seahorse_util_keylist_splice (GList *keys)
             prev->next = NULL;
             
             /* And return the new list */
-            return keys;
+            return objects;
         }
         
         psk = sk;
-        prev = keys;
+        prev = objects;
     }
     
     return NULL;
